defmodule Chronoscope.Application do
  # See https://hexdocs.pm/elixir/Application.html
  # for more information on OTP Applications
  @moduledoc false

  use Application

  @impl true
  def start(_type, _args) do
    children = [
      ChronoscopeWeb.Telemetry,
      {DNSCluster, query: Application.get_env(:chronoscope, :dns_cluster_query) || :ignore},
      {Phoenix.PubSub, name: Chronoscope.PubSub},
      # Start the Finch HTTP client for sending emails
      {Finch, name: Chronoscope.Finch},
      # Start a worker by calling: Chronoscope.Worker.start_link(arg)
      # {Chronoscope.Worker, arg},
      {DynamicSupervisor, name: Chronoscope.NTS.DynamicSupervisor, strategy: :one_for_one},
      {Task.Supervisor, name: Chronoscope.NTS.TaskSupervisor},
      {Registry, [keys: :unique, name: Chronoscope.NTS.Registry]},
      # Start to serve requests, typically the last entry
      ChronoscopeWeb.Endpoint
    ]

    # See https://hexdocs.pm/elixir/Supervisor.html
    # for other strategies and supported options
    opts = [strategy: :one_for_one, name: Chronoscope.Supervisor]
    Supervisor.start_link(children, opts)
  end

  # Tell Phoenix to update the endpoint configuration
  # whenever the application is updated.
  @impl true
  def config_change(changed, _new, removed) do
    ChronoscopeWeb.Endpoint.config_change(changed, removed)
    :ok
  end
end
